﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;
public class ControladorGeneral : MonoBehaviour
{
    #region variables publicas
    public int nivelSeleccionado;
    public int monedasActuales;
    public int nivelMaximoCompletado;
    public int totalNiveles;
    public bool firstTimeOpenApp;
    public List<int> skinsDesbloqueadas;
    public int skinActual;
    #endregion

    // Start is called before the first frame update
    void Start()
    {
        if (firstTimeOpenApp) //La primera vez que se abre la app (default values)
        {
            nivelMaximoCompletado = 1;
            nivelSeleccionado = 1;
            monedasActuales = 0;
            skinActual = 1;
            skinsDesbloqueadas.Add(1);
            firstTimeOpenApp = false;
        }
        LoadGame(); //Carga el juego en el caso de tener cosas guardadas
    }

    #region #FUNCIONES#

    #region #GET#
        //Devuelve el nivel maximo completado
        public int getNivelMaximo()
        {
            return nivelMaximoCompletado;
        }

        //Devuelve las monedas actuales
        public int getMonedas()
        {
            return monedasActuales;
        }

        //Devuelve el nivel seleccionado
        public int getNivelSeleccionado()
        {
            return nivelSeleccionado;
        }

        //Devuelve la skin actual
        public int getSkinActual()
        {
            return skinActual;
        }

        //Devuelve las skins desbloqueadas
        public List<int> getSkinsDesbloqueadas()
        {
            return skinsDesbloqueadas;
        }

        //Devuelve TRUE si la skin con skinId esta desbloqueada, devuelve FALSE si no esta desbloqueada
        public bool estaSkinDesbloqueada(int skinId)
        {
            if (skinsDesbloqueadas.Contains(skinId))
            {
                return true;
            }
            else
            {
                return false;
            }
        }

    #endregion

    #region #SET#

        //Actualiza el nivel seleccionado
        public void setNivel(int nivel)
        {
            nivelSeleccionado = nivel;
        }

        //Guarda la partida
        public void SaveGame()
        {
            saveSystem.SaveGame(this);
        }

        //Carga la partida
        public void LoadGame()
        {
            gameData data = saveSystem.LoadGame();

            nivelMaximoCompletado = data.maxLvlCompletado;
            monedasActuales = data.monedas;
            nivelSeleccionado = data.nivelSeleccionado;
            skinsDesbloqueadas = data.skinsDesbloqueadas;
            skinActual = data.skinActual;
            firstTimeOpenApp = data.firstTimeOpenApp;
        }

        //Suma monedas a las monedas actuales
        public void sumarMonedas(int monedas)
        {
            monedasActuales +=  monedas;
            SaveGame();
        }

        //Resta monedas a las monedas actuales
        public void restarMonedas(int monedasRestadas)
        {
            monedasActuales -= monedasRestadas;
            SaveGame();
        }

        //Pasa de nivel
        public void nextLvl()
        {
            if (nivelSeleccionado < totalNiveles) {
                nivelSeleccionado += 1;
            }
        }

        //Aumenta el nivel maximo completado
        public void desbloquearNivel()
        {
            if(nivelSeleccionado == nivelMaximoCompletado && nivelMaximoCompletado < totalNiveles)
            {
                nivelMaximoCompletado += 1;
            }
        }



        //Desbloquea skin segun su skinId
        public void desbloquearSkin(int skinId)
        {
            skinsDesbloqueadas.Add(skinId);
            SaveGame();
        }

        //Activa la skin segun su skinId
        public void setSkinActual(int skinId)
        {
            skinActual = skinId;
        }

    #endregion
    
    #endregion
}
