﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Movimiento_Jugador : MonoBehaviour
{

    #region variables privadas
    private float distancia = 0.4f; //Distancia entre cubos (posicion central)
    private float altura = 0.4f; //Altura base, el suelo mide 0.4
    private List<Vector3> movimientos = new List<Vector3>(); //Lista movimientos
    private float step;
    private float initialSpeed = 4f; //Velocidad inicial
    private float speed = 0f; //Velocida movimiento (TEMPORAL = 3)
    private float Acceleration = 1f; //Aceleracion
    private float maxSpeed = 20f; //Velocidad maxima
    private int maxMovimientosRecordar = 3; //Maximo de movimientos que guarda
    private float velocidadAccion = 0.5f; //Velocidad a la que se ejecuta una accion cuando el jugador esta parado (Ejemplo: El jugador encima un boton, el jugador encima la meta...)
    private bool enPlataforma = false; //Variable para controlar si el jugador esta encima una plataforma
    private List<string> movimientosPosibles = new List<string>(); //Guardo los tipos de suelo por donde puedo pasar
    private List<string> movimientosStop = new List<string>(); //Guardo los tipos de suelo donde me tengo que parar
    private float tamañoRaycast = 1f; //Medida raycast
    private bool puedoEjecutarAccion = true; //True = Se puede ejecutar accion
    private bool isMoving = false; //False = no se esta moviendo

    private Vector3 ultimaPosicion;
    private bool pausa;
    #endregion

    #region variables publicas

    public Vector2 startPos; 
    public Vector2 endPos;
    public Vector2 direction;

    public GameObject ControlMenus;
    public GameObject Niveles;
    public GameObject Sonidos;

    #endregion


    //Se ejecuta antes incluso que el Start!
    private void Awake()
    {
        //Añado los tags los cuales puedo pasar por encima con el player.
        movimientosPosibles.Add("Suelo");
        movimientosPosibles.Add("Boton");
        movimientosPosibles.Add("Meta");
        movimientosPosibles.Add("PlatFinal");
        movimientosPosibles.Add("Plataforma");

        //Añado los tags los cuales no puedo pasar por encima con el player.
        movimientosStop.Add("Pegamento");
        movimientosStop.Add("Pinchos");

        ultimaPosicion = transform.position;
        speed = initialSpeed;
        pausa = false;
    }

    // Update is called once per frame
    void Update()
    {
        altura = transform.position.y; //Guardo la altura actual del player (necesario por si subo/bajo de nivel)
        if (!pausa && ControlMenus.GetComponent<ControlMenu>().isGameStart())
        {
            teclaPulsada(); //Mira que telca se ha pulsado TECLADO (Herramienta debug)
            movimientoMovil(); //DETECCION MOVIMIENTO MOVIL 
            ejecutarMovimiento(); //Ejecuta el movimiento
            ejecutarAccion(); //Ejecuta una accion (si estoy parado)
        }
    }

    #region #FUNCIONES#
    private void movimientoMovil()
    {
        if (Input.touchCount > 0 && !enPlataforma)
        {
            Touch touch = Input.GetTouch(0);
            buscarDireccion(touch);
        }
    }

    private void buscarDireccion(Touch touch)
    {
        //docs.unity3d.com/ScriptReference/Touch-phase.html//
        switch (touch.phase)
        {
            case TouchPhase.Began: //Si acabo de pulsar la pantalla
                direction = new Vector2(0, 0); //Al pulsar la pantalla reinicio la direccion por si es solo un TAP.
                startPos = touch.position; //Guardo la posicion inicial
                break;

            case TouchPhase.Moved: //Si estoy arrastrando el dedo
                endPos = touch.position;
                direction = endPos - startPos; //Calculo la direccion
                break;

            case TouchPhase.Ended: //Si suelto

                Sonidos.GetComponent<ControlSonidos>().playMovimentSound();
                float x = direction.x;
                float y = direction.y;

                if (Mathf.Abs(x) > Mathf.Abs(y))
                {
                    if(x > 0)
                    {
                        buscarNuevaPosicion(new Vector3(distancia, 0, 0));    
                    }
                    else
                    {
                        buscarNuevaPosicion(new Vector3(-distancia, 0, 0));
                    }
                }
                else
                {
                    if(y > 0)
                    {
                        buscarNuevaPosicion(new Vector3(0, 0, distancia));
                    }
                    else if(y < 0)
                    {
                        buscarNuevaPosicion(new Vector3(0, 0, -distancia));
                    }

                }
                direction = new Vector2(0, 0); //Reinicio direccion para que no se mueva solo si he pasado de nivel.
                Debug.Log(direction);
                break;
        }
    }

    void teclaPulsada()
    {
        
        //Pre: Se llama en cada update
        //Post: Controla que boton se ha pulsado y guarda a la posicion donde se movera.
        if (!enPlataforma) { //Si la pelota no se esta en movimiento (subida en una plataforma)
            if (Input.GetKeyDown(KeyCode.LeftArrow)) //Si apreto a la izquierda
            {
                buscarNuevaPosicion(new Vector3(-distancia,0,0));
            }
            if (Input.GetKeyDown(KeyCode.RightArrow)) //Si apreto a la izquierda
            {
                buscarNuevaPosicion(new Vector3(distancia, 0, 0));
            }
            if (Input.GetKeyDown(KeyCode.UpArrow)) //Si apreto arriba
            {
                buscarNuevaPosicion(new Vector3(0, 0, distancia));
            }
            if (Input.GetKeyDown(KeyCode.DownArrow)) //Si apreto abajo
            {
                buscarNuevaPosicion(new Vector3(0, 0, -distancia));
            }
        }
    }   

    void buscarNuevaPosicion(Vector3 direccion)
    {
        //Pre: direccion = Direccion hacia donde se hace la comprobación de nueva posicion.
        //Post: Guarda en la lista movimientos la nueva posicion (si es que la hay)
        RaycastHit hit = new RaycastHit(); 
        RaycastHit lastHit = new RaycastHit();
        Vector3 direccionInicial = direccion;
        direccion.y += 0.5f; //Subo un poco para que el inicio del raycast quede mas arriba

        if (movimientos.Count == 0) //Estoy parado y me voy a mover
        {
            //Debug.DrawRay(transform.position + direccion, transform.TransformDirection(Vector3.down), Color.yellow, tamañoRaycast); //CONTROL DEVELOP
            if (Physics.Raycast(transform.position + direccion, transform.TransformDirection(Vector3.down), out hit, tamañoRaycast)) //Tiro Raycast en la direccion X y si tengo suelo entro en el if
            {
                if(hit.collider.tag != "Muro") { //Si hacia donde me intento mover no es un muro
                    while (movimientosPosibles.Contains(hit.collider.tag)) //Mientras sea suelo seguire mirando la siguiente posicion
                    {
                        lastHit = hit; //Guardo el ultimo hit hecho
                        //Debug.DrawRay(transform.position + direccion, transform.TransformDirection(Vector3.down), Color.yellow, tamañoRaycast); //CONTROL DEVELOP
                        if (Physics.Raycast(transform.position + direccion, transform.TransformDirection(Vector3.down), out hit, tamañoRaycast)) //Miro la sigiente posicion si esta libre y me puedo mover
                        {
                            direccion += direccionInicial;
                        }
                        else
                        {
                            break; //Salgo del while (por que si hit == null peta)
                        }
                    }
                    guardarMovimiento(hit, lastHit); //Guardo el movimiento
                }
            }
        }
        else //Si ya tengo movimientos guardados haré las comprobaciones desde la posicion guardada y no mi posicion actual
        {
            int totalMovimientos = movimientos.Count;
            Debug.DrawRay(movimientos[totalMovimientos-1] + direccion, transform.TransformDirection(Vector3.down), Color.yellow, tamañoRaycast); //CONTROL DEVELOP
            if (totalMovimientos < maxMovimientosRecordar && Physics.Raycast(movimientos[totalMovimientos - 1] + direccion, transform.TransformDirection(Vector3.down), out hit, tamañoRaycast)) //Si puedo recordar movimientos y Si tengo suelo en la direccion que intento
            {
                if (hit.collider.tag != "Muro") { //Si hacia donde me intento mover no es un muro
                    while (movimientosPosibles.Contains(hit.collider.tag)) //Mientras sea suelo seguire mirando la siguiente posicion
                    //while (hit.collider.tag == "Suelo" || hit.collider.tag == "Boton" || hit.collider.tag == "Meta" || hit.collider.tag == "PlatFinal") //Mientras esa suelo seguire mirando la siguiente posicion
                    {
                        lastHit = hit; //Guardo el ultimo hit hecho
                        Debug.DrawRay(movimientos[totalMovimientos - 1] + direccion, transform.TransformDirection(Vector3.down), Color.yellow, tamañoRaycast); //CONTROL DEVELOP
                        if (Physics.Raycast(movimientos[totalMovimientos - 1] + direccion, transform.TransformDirection(Vector3.down), out hit, tamañoRaycast)) //Miro la sigiente posicion si esta libre y me puedo mover
                        {
                            direccion += direccionInicial;
                        }
                        else
                        {
                            break; //Salgo del while (por que si hit es null peta)
                        }
                    }
                    guardarMovimiento(hit, lastHit);
                    //movimientos.Add(new Vector3(lastHit.transform.position.x, altura, lastHit.transform.position.z)); //Guardo el movimiento en lista
                }
            }
        }
        
    }

    private void guardarMovimiento(RaycastHit hit, RaycastHit lastHit)
    {
        //Pre: Hit = la ltima posicion mirada, lastHit = la penultima posicion mirada
        //Pots: Guardo elhit en la lista movimientos
        if (hit.collider == null ) //Si hit es null guardare la posicion anterior
        {
            movimientos.Add(new Vector3(lastHit.transform.position.x, altura, lastHit.transform.position.z)); //Guardo el movimiento en lista
           
        }
        else //Si hit no es null
        {
            if (movimientosStop.Contains(hit.collider.tag)) //Si hit es alguna casilla donde tengo que pararme me quedare encima
            {
                movimientos.Add(new Vector3(hit.transform.position.x, altura, hit.transform.position.z)); //Guardo el movimiento en lista
            }
            else //Si es una casilla la cual no puedo quedarme encima guardare la posicion anterior
            {
                movimientos.Add(new Vector3(lastHit.transform.position.x, altura, lastHit.transform.position.z)); //Guardo el movimiento en lista
            }
        }
    }

    private void ejecutarMovimiento()
    {
        //Pre: Se llama en cada update
        //Post: Ejecuta el movimiento de A a B de manera fluida

        if (movimientos.Count > 0)
        {
            if (speed <= maxSpeed) {
                speed = speed + Acceleration;
            }
        }
        else if(speed > initialSpeed)
        {
            speed = initialSpeed;
        }

        step = speed * Time.deltaTime;

        squashAndStretch(); //Ejecuto "animación"

        if (movimientos.Count > 0) { //Si tengo movimientos

            rotarSegunDireccion();
            //transform.position = Vector3.Lerp(transform.position, movimientos[0], 0.1f);
            transform.position = Vector3.MoveTowards(transform.position, movimientos[0], step); //MoveTowards mueve de punto A a punto B step by step (movimiento fluido)
            
            if (transform.position == movimientos[0]) //Si posicion actual es igual a la del movimineto[0] se elimina
            {
                movimientos.RemoveAt(0);
            }
        }
    }

    private void rotarSegunDireccion()
    {
        float margen = 0.1f;
        float rotacion;

        if(transform.position.z + margen < movimientos[0].z) //UP
        {
            rotacion = 0 - transform.rotation.y;
            transform.rotation = Quaternion.Euler(0, rotacion, 0);
        }
        if (transform.position.x - margen > movimientos[0].x) //LEFT
        {
            rotacion = 90 - transform.rotation.y;
            transform.rotation = Quaternion.Euler(0, -rotacion, 0);
        }
        
        if (transform.position.z - margen > movimientos[0].z) //DOWN
        {
            rotacion = 180 - transform.rotation.y;
            transform.rotation = Quaternion.Euler(0, rotacion, 0);
        }
        if (transform.position.x + margen < movimientos[0].x) //RIGHT
        {
            rotacion = 90 - transform.rotation.y;
            transform.rotation = Quaternion.Euler(0, rotacion, 0);
        }

    }

    private void squashAndStretch()
    {
        //float velocidadEscala = (speed - initialSpeed) * (5 / (maxSpeed - initialSpeed));
        float velocidadEscala = (speed - maxSpeed) * (10 / (initialSpeed - maxSpeed)); //FORMULA QUE TENGO EN PAPEL (para que el squash no se haga del 100% ya que si no desaparece el player)

        float squash = velocidadEscala / 20 + 0.5f;

        transform.localScale = new Vector3(0.4f * squash, 0.4f * squash, 0.4f);

    }

    private void ejecutarAccion()
    {
        if(movimientos.Count == 0 && puedoEjecutarAccion && !isMoving) //Si no tengo movimientos, puedo ejectuar acción y no me estoy moviendo
        {
            StartCoroutine("accion"); //Ejecuta corutina
        }
        else if(movimientos.Count != 0){ //Si me muevo
            puedoEjecutarAccion = true; //Podre activar otra accion
        }
    }

    IEnumerator accion()
    {
        yield return new WaitForSeconds(velocidadAccion); //Tiempo de espera antes de ejecutar la accion
        if(movimientos.Count == 0) //Comprobación extra para que no se ejecute la accion en medio de un movimiento (NECESARIA)
        { 
            activarAccion();
        }
        puedoEjecutarAccion = false; //Cuando activo una accion no puedo activar otra hasta que me haya movido de nuevo
    }

    private void activarAccion()
    {
        RaycastHit hit = new RaycastHit();
        Debug.DrawRay(transform.position, transform.TransformDirection(Vector3.down), Color.red, tamañoRaycast); //CONTROL DEVELOP
        Vector3 posRay = transform.position;
        posRay.y = posRay.y + 0.1f; //Para detectar el objeto que tiene debajo el player
        Physics.Raycast(posRay, transform.TransformDirection(Vector3.down), out hit, tamañoRaycast); //Rayo para abajo

        if(hit.collider.tag == "Boton") //Si el hit es boton ejectuo el script que tendra el boton
        {
            hit.collider.gameObject.GetComponent<Accion_Boton>().ActivarMovimiento();
        }
        else if(hit.collider.tag == "Plataforma" && !enPlataforma) //Si el hit es una Plataforma se movera hasta la siguiente posicion
        {
            enPlataforma = true;
            transform.SetParent(hit.collider.transform); //El player se engancha a la plataforma
            hit.collider.gameObject.GetComponentInParent<Movimiento_Plataforma>().mover();
        }
        else if(hit.collider.tag == "Meta") //Si el hit es la Meta se completa el nivel
        {
            //NIVEL COMPLETADO
            Sonidos.GetComponent<ControlSonidos>().playWinSound(); //Ejecuto Sonido Ganar
            ControlMenus.GetComponent<ControlMenu>().mostrarFinalNivel(); //Muestro pantalla final de nivel
            speed = 0;
            pausa = true;
        }
        StopCoroutine("accion"); //Detengo la acción
    }

    public void resetearMovimiento() //Funcion para reiniciar el movimiento
    {
        enPlataforma = false; //Deja de estar en plataforma
        pausa = false; //No esta en pausa
    }

    public void limpiarMovimientos()
    {
        movimientos.Clear(); //Elimino todos los movimientos de la lista
    }

    private void OnTriggerEnter(Collider other) //Control de eventos (cando entran en contacto triggers)
    {
        if(other.tag == "Pinchos") //En el caso de que choques con pinchos, has "perdido" y vuelves al inicio del nivel
        {
            ControlMenus.GetComponent<ControlMenu>().recargarElNivel();
        }
    }

    public void switchPausa() //Si estas en partida se pone pausa, si estas en pausa vuelves a la partida
    {
        if (!pausa)
        {
            pausa = true;
        }
        else
        {
            pausa = false;
        }
    }

    #endregion
}
